import json
import os
import sys
import time
from typing import Callable

import requests
from requests.structures import CaseInsensitiveDict

from entegywrapper.errors import EntegyInvalidAPIKeyError, EntegyNoDataError

sys.path.append(os.path.dirname(__file__))

API_ENDPOINTS = {
    "AU": "https://api.entegy.com.au",
    "US": "https://api-us.entegy.com.au",
    "EU": "https://api-eu.entegy.com.au",
}


class EntegyAPI:
    from .attendance_tracking.attendance_tracking import add_check_in, get_attended, get_attendees
    from .content.categories import (
        available_categories,
        create_categories,
        create_child_categories,
        delete_categories,
        deselect_categories,
        select_categories,
        update_category,
    )
    from .content.content import (
        add_children_content,
        create_content,
        delete_content,
        get_content,
        get_schedule_content,
        update_content,
    )
    from .content.documents import add_documents, add_external_content_documents
    from .content.multi_link import (
        add_multi_links,
        get_multi_links,
        remove_all_multi_links,
        remove_multi_link,
    )
    from .notification.notification import send_bulk_notification, send_notification
    from .plugins.ext_auth import external_authentication
    from .points.point_management import award_points, get_point_leaderboard, get_points
    from .profiles.profile_custom import (
        all_profile_custom,
        create_profile_custom,
        delete_profile_custom,
        get_profile_custom,
        update_profile_custom,
    )
    from .profiles.profile_links import (
        clear_profile_links,
        deselect_profile_links,
        multi_select_profile_links,
        page_profile_links,
        select_profile_link,
        selected_profile_links,
    )
    from .profiles.profile_payments import add_profile_payment
    from .profiles.profile_types import (
        all_profile_types,
        create_profile_type,
        delete_profile_type,
        get_profile_type,
        update_profile_type,
    )
    from .profiles.profiles import (
        all_profiles,
        create_profile,
        delete_profile,
        get_profile,
        send_welcome_email,
        sync_profile_block,
        sync_profiles,
        update_profile,
    )

    def __init__(
        self,
        api_key: str | list[str],
        api_secret: str | list[str],
        project_id: str,
        region: str = "AU",
    ):
        """
        Creates an Entegy API wrapper to interact with the specified project.

        Parameters
        ----------
            `api_key` (`str | list[str]`): Entegy API key(s)

            `api_secret` (`str | list[str]`): Entegy API secret key(s)

            `project_id` (`str`): Entegy project ID

            `region` (`str`, optional): project region: one of "AU", "US", "EU"; defaults to "AU"
        """
        if isinstance(api_key, list):
            assert isinstance(api_secret, list)
            assert len(api_key) == len(api_secret)
            assert all(isinstance(key, str) for key in api_key)
            assert all(isinstance(secret, str) for secret in api_secret)
        else:
            assert isinstance(api_key, str)
            assert isinstance(api_secret, str)
        assert isinstance(project_id, str)
        assert isinstance(region, str)
        assert region in API_ENDPOINTS.keys()

        if isinstance(api_key, list) and isinstance(api_secret, list):
            self.api_key = list(map(lambda x: x.strip(), api_key))
            self.api_secret = list(map(lambda x: x.strip(), api_secret))
        elif isinstance(api_key, list) or isinstance(api_secret, list):
            raise ValueError("API key and secret must both be lists or both be strings")
        else:
            self.api_key = api_key.strip()
            self.api_secret = api_secret.strip()

        self.current_key_pair = 0
        self.project_id = project_id.strip()

        self.headers = CaseInsensitiveDict()
        self.headers["Content-Type"] = "application/json"
        self.get_key()

        self.api_endpoint = API_ENDPOINTS[region]

    def get_key(self) -> str:
        """
        Returns the API Key. If a list of keys was provided, the current key is
        returned.

        Returns
        -------
            `str`: API Key
        """
        if isinstance(self.api_key, list):
            self.headers["Authorization"] = f"ApiKey {self.api_secret[self.current_key_pair]}"
            return self.api_key[self.current_key_pair]

        self.headers["Authorization"] = f"ApiKey {self.api_secret}"
        return self.api_key

    def cycle_key(self):
        """
        Cycles to the next API keypair, wrapping to the first where necessary.
        """
        self.current_key_pair = (self.current_key_pair + 1) % len(self.api_key)

    def get_endpoint(self) -> str:
        """
        Returns the endpoint URL.

        Returns
        -------
            `str`: API endpoint URL
        """
        return self.api_endpoint

    def request(self, method: Callable, endpoint: str, data: dict) -> dict:
        """
        Sends the given data to the given endpoint of the Entegy API, using the given
        method. Internalised to allow for automatic key cycling and error handling.

        Parameters
        ----------
            `method` (`Callable`): method to use to send the request

            `endpoint` (`str`): API endpoint to which to post

            `data` (`dict`): data to post

        Raises
        ------
            `EntegyInvalidAPIKeyError`: if the API keys are invalid

        Returns
        -------
            `dict`: response data
        """
        keys_attempted = 0
        failed_requests = 0

        data |= {"apiKey": self.get_key(), "projectId": self.project_id}

        response = None
        while response is None:
            response = method(endpoint, headers=self.headers, data=json.dumps(data))

            try:
                response = response.json()
            except:
                failed_requests += 1
                if failed_requests >= 5:
                    raise EntegyNoDataError()

                response = None
                continue

            match response["response"]:
                case 403:  # invalid API key
                    failed_requests += 1
                    if failed_requests >= 5:
                        raise EntegyInvalidAPIKeyError()

                    response = None
                case 489:  # rate-limit
                    if keys_attempted >= len(self.api_key):
                        duration = response["resetDuration"]
                        print(f"Rate limit reached, waiting {duration} seconds")
                        time.sleep(duration + 2)
                        print("Continuing...")
                        keys_attempted = 0
                        response = None
                    else:
                        self.cycle_key()
                        key = self.get_key()
                        data["apiKey"] = key
                        print(f"Rate limit reached, trying alternate key: {key}")
                        keys_attempted += 1
                        response = None

        return response

    def post(self, endpoint: str, data: dict) -> dict:
        """
        Posts the given data to the given endpoint of the Entegy API.

        Parameters
        ----------
            `endpoint` (`str`): API endpoint to which to post

            `data` (`dict`): data to post

        Returns
        -------
            `dict`: response data
        """
        return self.request(requests.post, endpoint, data)

    def delete(self, endpoint: str, data: dict) -> dict:
        """
        Deletes the given data from the given endpoint of the Entegy API.

        Parameters
        ----------
            `endpoint` (`str`): API endpoint from which to delete

            `data` (`dict`): data to delete

        Returns
        -------
            `dict`: response data
        """
        return self.request(requests.delete, endpoint, data)



from enum import Enum
from typing import Any, Optional

from pydantic import BaseModel

from .project import ProjectEventInfo, ProjectStatus, ProjectType, SoftwareElement


class ProfileExtendedPrivacy(Enum):
    PUBLIC = "Public"
    CONNECTIONS = "Connections"
    HIDDEN = "Hidden"


class Permissions(BaseModel):
    loggedInApp: Optional[bool] = None
    loggedInCapture: Optional[bool] = None
    showInList: Optional[bool] = None
    allowMessaging: Optional[bool] = None
    showEmail: Optional[ProfileExtendedPrivacy] = None
    showContactNumber: Optional[ProfileExtendedPrivacy] = None
    apiManaged: Optional[bool] = None
    printedBadge: Optional[bool] = None
    optedOutOfEmails: Optional[bool] = None
    acceptedTerms: Optional[bool] = None


class ProfileReference(BaseModel):
    profileId: Optional[str] = None
    externalReference: Optional[str] = None
    internalReference: Optional[str] = None
    badgeReference: Optional[str] = None
    secondaryId: Optional[str] = None


class Profile(BaseModel):
    type: str
    firstName: str
    lastName: str
    profileId: Optional[str] = None
    externalReference: Optional[str] = None
    internalReference: Optional[str] = None
    badgeReference: Optional[str] = None
    accessCode: Optional[str] = None  # ^[A-Za-z0-9]+(?:[._-][A-Za-z0-9]+)*$
    password: Optional[str] = None
    title: Optional[str] = None
    displayName: Optional[str] = None
    organization: Optional[str] = None
    position: Optional[str] = None
    email: Optional[str] = None
    contactNumber: Optional[str] = None
    imageUrl: Optional[str] = None
    created: Optional[str] = None
    lastUpdated: Optional[str] = None
    enabled: Optional[bool] = None
    permissions: Optional[Permissions] = None
    customFields: Optional[dict[str, Any]] = None
    parentProfile: Optional[ProfileReference] = None


class ProfileType(BaseModel):
    name: str
    isOrganiser: bool
    allowAppLogin: bool
    price: int
    moduleId: int


class CustomProfileFieldType(Enum):
    MULTI_CHOICE = "MultiChoice"
    SHORT_TEXT = "ShortText"
    MEDIUM_TEXT = "MediumText"
    LONG_TEXT = "LongText"
    FACEBOOK = "Facebook"
    TWITTER = "Twitter"
    INSTAGRAM = "Instagram"
    WEBSITE = "Website"


class MultiChoiceOptions(BaseModel):
    optionId: int
    name: str
    externalMappings: str


class CustomProfileField(BaseModel):
    key: str
    name: str
    required: bool
    userAccess: str
    profileVisibility: str
    type: CustomProfileFieldType
    sortOrder: Optional[int] = None
    externallyManaged: bool
    options: Optional[list[MultiChoiceOptions]]


class ProfileCreate(BaseModel):
    externalReference: str
    projectName: str
    projectShortName: str
    eventCode: str
    renewalDate: str
    status: ProjectStatus
    type: ProjectType
    softwareElements: list[SoftwareElement]
    eventInfo: ProjectEventInfo


class ProfileUpdate(BaseModel):
    type: Optional[str] = None
    firstName: Optional[str] = None
    lastName: Optional[str] = None
    externalReference: Optional[str] = None
    badgeReference: Optional[str] = None
    accessCode: Optional[str] = None  # ^[A-Za-z0-9]+(?:[._-][A-Za-z0-9]+)*$
    password: Optional[str] = None
    title: Optional[str] = None
    organization: Optional[str] = None
    position: Optional[str] = None
    email: Optional[str] = None
    contactNumber: Optional[str] = None
    imageUrl: Optional[str] = None
    enabled: Optional[bool] = None
    permissions: Optional[Permissions] = None
    customFields: Optional[dict[str, Any]] = None


class ProfileIdentifier(Enum):
    PROFILE_ID = "profileId"
    EXTERNAL_REFERENCE = "externalReference"
    INTERNAL_REFERENCE = "internalReference"
    BADGE_REFERENCE = "badgeReference"


class PaymentStatus(Enum):
    PENDING = "Pending"
    CANCELLED = "Cancelled"
    PAID = "Paid"
    REFUNDED = "Refunded"


class PaymentMethod(Enum):
    NONE = "None"
    CREDIT_CARD = "CreditCard"
    DIRECT_DEPOSIT = "DirectDeposit"
    CASH = "Cash"
    CHEQUE = "Cheque"
    OTHER = "Other"


class PaymentInfo(BaseModel):
    profileId: str
    externalReference: str
    internalReference: str
    badgeReference: str
    currency: str
    amount: int
    description: Optional[str] = None
    amountTax: Optional[int] = None
    amountTaxRate: Optional[float] = None
    platformFee: Optional[int] = None
    platformFeeTax: Optional[int] = None
    platformFeeTaxRate: Optional[float] = None
    platformFeeInvoiceId: Optional[str] = None
    transactionId: Optional[str] = None
    gateway: Optional[str] = None
    gatewayAccountId: Optional[str] = None
    status: Optional[PaymentStatus] = None
    method: Optional[PaymentMethod] = None



from enum import Enum


class PageSetting(Enum):
    HIDDEN_IN_APP = "hiddenInApp"
    UNCLICKABLE = "unclickable"
    DISABLE_COMMENTS = "disableComments"
    DISABLE_RATING = "disableRating"
    REQUIRE_LOGIN = "requireLogin"
    REMINDER_ALERT = "reminderAlert"
    API_MANAGED = "apiManaged"
    SHOW_BY_PROFILE = "showByProfile"



from enum import Enum
from typing import Optional

from pydantic import BaseModel


class ApiKeyPermission(Enum):
    VIEW_CONTENT = "ViewContent"
    EDIT_CONTENT = "EditContent"
    EDIT_PROFILES = "EditProfiles"
    VIEW_PROFILES = "ViewProfiles"
    ACHIEVEMENTS = "Achievements"
    SEND_NOTIFICATIONS = "SendNotifications"


class Region(Enum):
    AU = "61a948f2-d505-4b0b-81de-31af6925647e"
    US = "2b9bd3fc-405e-4df5-888d-f5323e2b5093"
    EU = "86f89b50-1bbb-4019-9ca2-b2d9f4167064"


class ProjectEventInfo(BaseModel):
    startDate: str
    endDate: str


class ProjectType(Enum):
    EVENT = "Event"
    ONGOING = "Ongoing"
    DEMO = "Demo"
    PORTAL = "Portal"
    DEMO_TEMPLATE = "DemoTemplate"


class ProjectStatus(Enum):
    DRAFT = "Draft"
    HAND_OVER = "HandOver"
    POPULATE_AND_TESTING = "PopulateAndTesting"
    PRODUCTION = "Production"
    FINISHED = "Finished"
    EXPIRED = "Expired"
    CANCELED = "Canceled"


class SoftwareElement(Enum):
    APP = "App"
    STORE_LISTING = "StoreListing"
    ENGAGE = "Engage"
    CAPTURE = "Capture"
    TRACK = "Track"
    INTERACT = "Interact"
    REGISTRATION = "Registration"
    MARKET = "Market"
    KIOSK = "Kiosk"
    KIOSK_ADDITIONAL = "KioskAdditional"
    EMAIL_DOMAIN = "EmailDomain"
    FLOOR_PLAN = "FloorPlan"


class Project(BaseModel):
    projectId: Optional[str] = None
    regionId: Optional[Region] = None
    regionName: Optional[str] = None
    externalReference: Optional[str] = None
    internalReference: Optional[str] = None
    projectName: Optional[str] = None
    projectShortName: Optional[str] = None
    iconUrl: Optional[str] = None
    eventCode: Optional[str] = None
    renewalDate: Optional[str] = None
    status: Optional[ProjectStatus] = None
    type: Optional[ProjectType] = None
    softwareElements: Optional[list[SoftwareElement]] = None
    eventInfo: Optional[ProjectEventInfo] = None


class ProjectApiKey(BaseModel):
    apiKeyId: str
    description: str
    expireDate: str
    allowedDomains: list[str]
    permissions: list[ApiKeyPermission]



class EntegyError(Exception):
    """Base class for all Entegy errors."""

    pass


class EntegyFailedRequestError(EntegyError):
    """Raised when the Entegy API returns a failed request response."""

    pass


class EntegyInvalidAPIKeyError(EntegyError):
    """Raised when the Entegy API returns an invalid API key response."""

    pass


class EntegyNoDataError(EntegyError):
    """Raised when the Entegy API responds with no data."""

    pass


class EntegyServerError(EntegyError):
    """Raised when the Entegy API returns a server error response."""

    pass



from pydantic import BaseModel

from .content import NamedLink
from .profile import Profile


class ExhibitorLead(BaseModel):
    Profile: Profile
    scannedTime: str
    syncedTime: str


class ProfileLead(BaseModel):
    exhibitor: NamedLink
    scannedTime: str
    syncedTime: str



from __future__ import annotations

from typing import TYPE_CHECKING

from entegywrapper.errors import EntegyFailedRequestError

if TYPE_CHECKING:
    from entegywrapper import EntegyAPI


def external_authentication(self: EntegyAPI, profile_id: str, device_id: str) -> dict:
    """
    Authenticates a user's login via an external system.

    Parameters
    ----------
        `profile_id` (`str`): the profileId that is logging in

        `device_id` (`int`): the deviceId that is logging in

    Returns
    -------
        `dict`: the response from the API
    """
    data = {"profileId": profile_id, "deviceId": device_id, "requestVersion": 1}

    response = self.post(self.api_endpoint + "/v2/Authentication/ExternalProfile", data=data)

    match response["response"]:
        case 200:
            return response
        case 401:
            raise EntegyFailedRequestError("Profile Doesn't Exist")
        case 406:
            raise EntegyFailedRequestError("Invalid Device Id")
        case 407:
            raise EntegyFailedRequestError("Profile is Banned")
        case 408:
            raise EntegyFailedRequestError("Profile is not Active")
        case _:
            raise EntegyFailedRequestError(
                f"{response['response']}: {response.get('message', 'Unknown error')}"
            )



from pydantic import BaseModel

from .content import NamedLink
from .profile import Profile


class Attendee(BaseModel):
    profile: Profile
    checkInTime: str


class Attended(BaseModel):
    session: NamedLink
    checkInTime: str



from enum import Enum, IntEnum
from typing import Optional

from pydantic import BaseModel

from .page_settings import PageSetting


class TemplateType(Enum):
    ABOUT = "About"
    ABSTRACT = "Abstract"
    ABSTRACTS = "Abstracts"
    EXHIBITOR = "Exhibitor"
    EXHIBITORS = "Exhibitors"
    FLOOR_PLAN = "FloorPlan"
    GENERIC_GROUP = "GenericGroup"
    GENERIC_GROUP_PAGE = "GenericGroupPage"
    HTML_GROUP = "HTMLGroup"
    HTML_PAGE = "HTMLPage"
    ROOM = "Room"
    SCHEDULE = "Schedule"
    SCHEDULE_DAY = "ScheduleDay"
    SESSION = "Session"
    SESSION_GROUP = "SessionGroup"
    SESSION_SEGEMENT = "SessionSegement"
    SESSION_TYPE = "SessionType"
    SPEAKER = "Speaker"
    SPEAKERS = "Speakers"
    SPONSOR = "Sponsor"
    SPONSORS = "Sponsors"
    STREAM = "Stream"


class Icon(IntEnum):
    """
    All icons can be found here: https://situ.entegy.com.au/Docs/Api/document-icons.
    """

    FACEBOOK = 1
    LINKEDIN = 2
    TWITTER = 3
    MAPPIN = 4
    GLOBE = 5
    ENEVELOPE = 6
    PHONE = 7
    SPEAKER = 8
    FILMREEL = 9
    DESKTOP = 10
    CHAT = 11
    MAP = 12
    ARTICLE = 13
    GALLERY = 14
    OUTBOX = 15
    LINK = 16
    PIN = 17
    BARGRAPH = 18
    PAPERCLIP = 19
    ACCESSPOINT = 20
    CALENDAR = 21
    CHECKMARK = 22
    PAPERPLANE = 23
    STAR = 24
    WARNINGTRIANGLE = 25
    SEARCH = 26
    FOLDER = 27
    INBOX = 28
    EDIT = 31
    PEOPLE = 32
    DOCUMENT = 35
    HOME = 37
    HOMESEARCH = 38
    DINER = 39
    OPENSIGN = 40
    INFOCIRCLE = 41
    HELPCIRCLE = 42
    FLAG = 43
    WINEGLASS = 44
    COCKTAIL = 45
    COFFEE = 46
    NEWS = 47
    CAR = 48
    OFFICEBUILDING = 49
    INFOSQUARE = 52
    GLOBESEARCH = 53
    ALARMCLOCK = 54
    GUITAR = 55
    ROADSIGNS = 56
    WIFITABLET = 57
    OPENBOOK = 58
    XBOX = 62
    GEAR = 64
    LOGOUT = 65
    INSTAGRAM = 68
    GOOGLEPLUS = 69
    HASHTAG = 70
    BACK = 71
    FORWARD = 72
    REFRESH = 73
    DOCUMENTSTAMP = 75
    ALARMBELL = 77
    ALARMBELLCOG = 78


class Document(BaseModel):
    name: str
    externalReference: str
    icon: Icon
    fileUrl: str


class ExternalContent(BaseModel):
    name: str
    externalReference: str
    icon: Icon
    fileUrl: str
    type: str


class Link(BaseModel):
    templateType: TemplateType
    moduleId: int
    externalReference: str


class NamedLink(Link):
    name: str


class Category(BaseModel):
    moduleId: int
    externalReference: str
    name: Optional[str] = None
    childCategories: list["Category"] = []


class Content(BaseModel):
    contentType: str
    templateType: TemplateType
    moduleId: int
    externalReference: str
    mainImage: str
    strings: dict[str, str]
    pageSettings: Optional[dict[PageSetting, bool]] = None
    sortOrder: Optional[int] = None
    documents: Optional[list[Document]] = None
    links: Optional[list[Link]] = None
    multiLinks: Optional[list[NamedLink]] = None
    selectedCategories: Optional[list[Category]] = None
    children: Optional[list["Content"]] = None


class ContentChildCreate(BaseModel):
    name: str
    externalReference: Optional[str] = None
    mainImage: Optional[str] = None
    strings: Optional[dict[str, str]] = None
    links: Optional[list[Link]] = None
    sortOrder: Optional[int] = None



from .content import Category, Content, Document, Link, NamedLink


class ScheduleDay(Content):
    children: list[Content]


class Schedule(Content):
    days: list[ScheduleDay]


class SessionSegment(Content):
    links: list[Link]
    multiLinks: list[NamedLink]
    documents: list[Document]


class Session(Content):
    links: list[Link]
    multiLinks: list[NamedLink]
    documents: list[Document]
    selectedCategories: list[Category]
    segments: list[SessionSegment]


class SessionGroup(Content):
    documents: list[Document]
    links: list[Link]
    multiLinks: list[NamedLink]
    selectedCategories: list[Category]



from __future__ import annotations

from typing import TYPE_CHECKING, Optional

from entegywrapper.errors import EntegyFailedRequestError
from entegywrapper.schemas.attendance_tracking import Attended, Attendee

if TYPE_CHECKING:
    from entegywrapper import EntegyAPI


def add_check_in(
    self: EntegyAPI,
    *,
    profile_id: Optional[str] = None,
    profile_external_reference: Optional[str] = None,
    profile_internal_reference: Optional[str] = None,
    profile_badge_reference: Optional[str] = None,
    session_module_id: Optional[str] = None,
    session_external_reference: Optional[str] = None,
) -> bool:
    """
    Checks the specified profile into the specified session.

    Parameters
    ----------
        `profile_id` (`str`, optional): the profileId of the profile; defaults to `None`

        `profile_external_reference` (`str`, optional): the externalReference of the profile; defaults to `None`

        `profile_badge_reference` (`str`, optional): the badgeReference of the profile; defaults to `None`

        `profile_internal_reference` (`str`, optional): the internalReference of the profile; defaults to `None`

        `session_module_id` (`str`, optional): the moduleId of the session; defaults to `None`

        `session_external_reference` (`str`, optional): the externalReference of the session; defaults to `None`

    Raises
    ------
        `ValueError`: if the profile or session identifier is not specified

        `EntegyFailedRequestError`: if the API request fails

    Returns
    -------
        `bool`: whether the check-in was successful
    """
    data = {}

    if profile_id is not None:
        data["profileReference"] = {"profileId": profile_id}
    elif profile_external_reference is not None:
        data["profileReference"] = {"externalReference": profile_external_reference}
    elif profile_badge_reference is not None:
        data["profileReference"] = {"badgeReference": profile_badge_reference}
    elif profile_internal_reference is not None:
        data["profileReference"] = {"internalReference": profile_internal_reference}
    else:
        raise ValueError("Please specify a profile identifier")

    if session_module_id is not None:
        data["contentReference"] = {"moduleId": session_module_id}
    elif session_external_reference is not None:
        data["contentReference"] = {"externalReference": session_external_reference}
    else:
        raise ValueError("Please specify a session identifier")

    response = self.post(self.api_endpoint + "/Track/AddCheckIn", data=data)

    match response["response"]:
        case 200:
            return True
        case 401:
            raise EntegyFailedRequestError("Missing content reference")
        case 402:
            raise EntegyFailedRequestError("Missing profile reference")
        case _:
            raise EntegyFailedRequestError(
                f"{response['response']}: {response.get('message', 'Unknown error')}"
            )


def get_attendees(
    self: EntegyAPI,
    *,
    module_id: Optional[str] = None,
    external_reference: Optional[str] = None,
) -> list[Attendee]:
    """
    Returns the attendees who attended the session specified by the given
    identifier.

    Parameters
    ----------
        `module_id` (`str`, optional): the moduleId of the session; defaults to `None`

        `external_reference` (`str`, optional): the externalReference of the session; defaults to `None`

    Raises
    ------
        `EntegyFailedRequestError`: if the API request fails

    Returns
    -------
        `list[Attendee]`: the attendees who attended the session
    """
    data = {}

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/Track/Attendees", data=data)

    if response["response"] == 401:
        raise EntegyFailedRequestError("Missing profile reference")

    return [Attendee(**attendee) for attendee in response["attendees"]]


def get_attended(
    self: EntegyAPI,
    *,
    profile_id: Optional[str] = None,
    external_reference: Optional[str] = None,
    badge_reference: Optional[str] = None,
    internal_reference: Optional[str] = None,
) -> list[Attended]:
    """
    Returns the sessions attended by the profile specified by the given
    identifier.

    Parameters
    ----------
        `profile_id` (`str`, optional): the profileId of the profile; defaults to `None`

        `external_reference` (`str`, optional): the externalReference of the profile; defaults to `None`

        `badge_reference` (`str`, optional): the badgeReference of the profile; defaults to `None`

        `internal_reference` (`str`, optional): the internalReference of the profile; defaults to `None`

    Raises
    ------
        `EntegyFailedRequestError`: if the API request fails

    Returns
    -------
        `list[Attended`: the sessions attended by the specified user
    """
    data = {}

    if profile_id is not None:
        data["profileId"] = profile_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    elif badge_reference is not None:
        data["badgeReference"] = badge_reference
    elif internal_reference is not None:
        data["internalReference"] = internal_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/Track/Attended", data=data)

    if response["response"] == 401:
        raise EntegyFailedRequestError("Missing profile reference")

    return [Attended(**attended) for attended in response["sessions"]]



from __future__ import annotations

import os
import sys
from typing import TYPE_CHECKING, Optional

from entegywrapper.errors import EntegyFailedRequestError
from entegywrapper.schemas.content import Document, ExternalContent, TemplateType

if TYPE_CHECKING:
    from entegywrapper import EntegyAPI

sys.path.append(os.path.dirname(__file__))


def add_documents(
    self: EntegyAPI,
    template_type: TemplateType,
    file_documents: list[Document],
    module_id: Optional[int] = None,
    external_reference: Optional[str] = None,
):
    """
    Adds documents to a page.

    Parameters
    ----------
        `template_type` (`TemplateType`): the templateType of the page to add the documents to

        `file_documents` (`list[Document]`): the file documents to add

        `module_id` (`int`): the moduleId of the page to add the documents to

        `external_reference` (`str`): the externalReference of the page to add the documents to

    Raises
    ------
        `ValueError`: if no identifier is specified

        `EntegyFailedRequestError`: if the API request fails
    """
    data = {
        "templateType": template_type,
        "fileDocuments": file_documents,
    }

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/Document/AddFile", data=data)

    match response["response"]:
        case 200:
            return
        case _:
            raise EntegyFailedRequestError(
                f"{response['response']}: {response.get('message', 'Unknown error')}"
            )


def add_external_content_documents(
    self: EntegyAPI,
    template_type: TemplateType,
    external_content_items: list[ExternalContent],
    module_id: Optional[int] = None,
    external_reference: Optional[str] = None,
):
    """
    Adds external content documents to a page.

    Parameters
    ----------
        `template_type` (`TemplateType`): the templateType of the page to add the documents to

        `external_content_items` (`list[ExternalContent]`): the external content documents to add

        `module_id` (`int`): the moduleId of the page to add the documents to

        `external_reference` (`str`): the externalReference of the page to add the documents to

    Raises
    ------
        `ValueError`: if no identifier is specified

        `EntegyFailedRequestError`: if the API request fails
    """
    data = {
        "templateType": template_type,
        "externalContentItems": external_content_items,
    }

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/Document/AddExternalContent", data=data)

    match response["response"]:
        case 200:
            return
        case _:
            raise EntegyFailedRequestError(
                f"{response['response']}: {response.get('message', 'Unknown error')}"
            )



from __future__ import annotations

from typing import TYPE_CHECKING, Optional

from entegywrapper.errors import EntegyFailedRequestError, EntegyNoDataError
from entegywrapper.schemas.content import Content, ContentChildCreate, TemplateType
from entegywrapper.schemas.schedule import Schedule

if TYPE_CHECKING:
    from entegywrapper import EntegyAPI


def get_content(
    self: EntegyAPI,
    template_type: TemplateType,
    *,
    module_id: Optional[int] = None,
    external_reference: Optional[str] = None,
    include_categories: bool = False,
    include_documents: bool = False,
    include_links: bool = False,
    include_multi_links: bool = False,
    include_page_settings: bool = False,
) -> Content:
    """
    Returns an entire schedule.

    Parameters
    ----------
        `template_type` (`TemplateType`): the templateType of the content

        `module_id` (`int`, optional): the moduleId of the content; defaults to `None`

        `external_reference` (`str`, optional): the externalReference of the content; defaults to `None`

        `include_categories` (`bool`, optional): whether to include Categories in the response; defaults to `False`

        `include_documents` (`bool`, optional): whether to include Documents in the response; defaults to `False`

        `include_links` (`bool`, optional): whether to include Links in the response; defaults to `False`

        `include_multi_links` (`bool`, optional): whether to include MultiLinks in the response; defaults to `False`

        `include_page_settings` (`bool`, optional): whether to include PageSettings in the response; defaults to `False`

    Raises
    ------
        `ValueError`: if no identifier is specified

        `EntegyFailedRequestError`: if the API request fails

    Returns
    -------
        `dict`: API response JSON
    """
    data = {
        "templateType": template_type,
        "includeCategories": include_categories,
        "includeDocuments": include_documents,
        "includeLinks": include_links,
        "includeMultiLinks": include_multi_links,
        "includePageSettings": include_page_settings,
    }

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/Content", data=data)

    if "content" not in response:
        raise EntegyNoDataError("No content returned")

    return Content(**response["content"])


def get_schedule_content(
    self: EntegyAPI,
    *,
    module_id: Optional[int] = None,
    external_reference: Optional[str] = None,
    include_categories: bool = False,
    include_documents: bool = False,
    include_links: bool = False,
    include_multi_links: bool = False,
    include_page_settings: bool = False,
) -> Schedule:
    """
    Returns an entire schedule.

    Parameters
    ----------
        `module_id` (`int`, optional): the moduleId of the schedule; defaults to `None`

        `external_reference` (`str`, optional): the externalReference of the schedule; defaults to `None`

        `include_categories` (`bool`, optional): whether to include Categories in the response; defaults to `False`

        `include_documents` (`bool`, optional): whether to include Documents in the response; defaults to `False`

        `include_links` (`bool`, optional): whether to include Links in the response; defaults to `False`

        `include_multi_links` (`bool`, optional): whether to include MultiLinks in the response; defaults to `False`

        `include_page_settings` (`bool`, optional): whether to include PageSettings in the response; defaults to `False`

    Raises
    ------
        `ValueError`: if no identifier is specified

        `EntegyFailedRequestError`: if the API request fails

    Returns
    -------
        `Schedule`: the schedule
    """
    data = {
        "includeCategories": include_categories,
        "includeDocuments": include_documents,
        "includeLinks": include_links,
        "includeMultiLinks": include_multi_links,
        "includePageSettings": include_page_settings,
    }

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/Content/Schedule", data=data)

    if "content" not in response:
        raise EntegyNoDataError("No content returned")

    return Schedule(**response["content"])


def create_content(self: EntegyAPI, content: Content, *, content_group: str = "Default") -> int:
    """
    Creates a root content item.

    Parameters
    ----------
        `content` (`Content`): the content to create

        `content_group` (`str`, optional) the content group in the core this new root content should go in; defaults to "Default"

    Raises
    ------
        `EntegyFailedRequestError`: if the API request fails
    """
    data = {"contentGroup": content_group, "content": content}

    response = self.post(self.api_endpoint + "/v2/Content/Create", data=data)

    match response["response"]:
        case 200:
            return response["moduleId"]
        case 401:
            raise EntegyFailedRequestError("Missing or invalid template type")
        case 402:
            raise EntegyFailedRequestError("Duplicate External Reference")
        case 404:
            raise EntegyFailedRequestError("Missing Name")
        case _:
            raise EntegyFailedRequestError(
                f"{response['response']}: {response.get('message', 'Unknown error')}"
            )


def add_children_content(
    self: EntegyAPI,
    template_type: TemplateType,
    child_template_type: TemplateType,
    children: list[ContentChildCreate],
    *,
    module_id: Optional[int] = None,
    external_reference: Optional[str] = None,
):
    """
    Adds children to templateType.

    Parameters
    ----------
        `template_type` (`string`): the templateType the children are being added to

        `child_template_type` (`string`): the templateType for the children to create

        `children` (`list[ContentChildCreate]`): the page data to add to the root templateType

        `module_id` (`int`, optional): the name for the page; defaults to `None`

        `external_reference` (`str`, optional): the externalReference for the page; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified

        `EntegyFailedRequestError`: if the API request fails
    """
    data = {
        "templateType": template_type,
        "childTemplateType": child_template_type,
        "children": children,
    }

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/Content/AddChildren", data=data)

    match response["response"]:
        case 200:
            return
        case 401:
            raise EntegyFailedRequestError("Missing ID")
        case 402:
            raise EntegyFailedRequestError("Page doesn't exist")
        case 404:
            raise EntegyFailedRequestError("Invalid Child Template ID")
        case 405:
            raise EntegyFailedRequestError("Missing Children")
        case 406:
            raise EntegyFailedRequestError("Duplicate External Reference")
        case _:
            raise EntegyFailedRequestError(response)


def update_content(
    self: EntegyAPI,
    template_type: TemplateType,
    content: Content,
    *,
    module_id: Optional[int] = None,
    external_reference: Optional[str] = None,
):
    """
    Updates data within a content item.

    Parameters
    ----------

        `template_type` (`TemplateType`): the templateType to update

        `content` (`Content`): the content to update

        `module_id` (`int`, optional): the moduleId of the page to update; defaults to `None`

        `external_reference` (`str`, optional): the externalReference of the page to update; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified

        `EntegyFailedRequestError`: if the API request fails
    """
    data = {"templateType": template_type, "content": content}

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/Content/Update", data=data)

    match response["response"]:
        case 200:
            return
        case 400:
            raise EntegyFailedRequestError("Content not provided")
        case 401:
            raise EntegyFailedRequestError("Content doesn't exist")
        case 406:
            raise EntegyFailedRequestError("Invalid page setting")
        case 410:
            raise EntegyFailedRequestError("Invalid String")
        case 413:
            raise EntegyFailedRequestError("Link doesn't exist")


def delete_content(
    self: EntegyAPI,
    template_type: TemplateType,
    *,
    module_id: Optional[int] = None,
    external_reference: Optional[str] = None,
):
    """
    Deletes a content resource from the Entegy System. Any content deleted is
    unrecoverable.

    WARNING
    -------
        Deleting root content will result in all child pages being deleted.

    Parameters
    ----------
        `template_type` (`TemplateType`): the templateType of the resource to delete

        `module_id` (`int`, optional): the moduleId of the page to delete; defaults to `None`

        `external_reference` (`str`, optional): the externalReference of the page to delete; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified

        `EntegyFailedRequestError`: if the API request fails
    """
    data = {"templateType": template_type}

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.delete(self.api_endpoint + "/v2/Content/Delete", data=data)

    match response["response"]:
        case 200:
            return
        case 401:
            raise EntegyFailedRequestError("Missing ID")
        case _:
            raise EntegyFailedRequestError(
                f"{response['response']}: {response.get('message', 'Unknown error')}"
            )



from enum import Enum

from pydantic import BaseModel

from .profile import Profile


class PointType(Enum):
    COMMENT = "Comment"
    COMMENT_WITH_IMAGE = "CommentWithImage"
    STATUS = "Status"
    STATUS_WITH_IMAGE = "StatusWithImage"
    VIEW_COMMENT = "ViewComment"
    PROFILE_LOGIN = "ProfileLogin"
    PROFILE_UPDATED = "ProfileUpdated"
    PROFILE_IMAGE_UPDATED = "ProfileImageUpdated"
    VIEW_PAGE = "ViewPage"
    VIEW_PAGE_FIRST_TIME = "ViewPageFirstTime"
    VIEW_NOTIFICATION = "ViewNotification"
    MESSAGE_SENT = "MessageSent"
    FEEDBACK_SUBMITTED = "FeedbackSubmitted"
    LEAD_CREATED = "LeadCreated"
    SESSION_TRACKED = "SessionTracked"
    INTERACTIVE_SESSION_VOTE = "InteractiveSessionVote"
    INTERACTIVE_SESSION_COMMENT = "InteractiveSessionComment"
    INTERACTIVE_SESSION_QUESTION = "InteractiveSessionQuestion"
    MANUAL_POINTS = "ManualPoints"


class Achievement(BaseModel):
    achievementId: int
    title: str
    message: str
    pointType: PointType
    pointOccurrancesRequired: int
    pointReward: int
    iconUrl: str


class AchievementUnlocked(Achievement):
    unlockedTime: str


class LeaderboardPosition(BaseModel):
    profile: Profile
    position: int
    points: int
    unlockedAchievementsCount: int
